declare interface Applicative<A> extends Apply<A> {
    of(a: A): Applicative<A>;
}

declare interface Apply<A> extends Functor<A> {
    ap<B, C>(j: Apply<B>): Apply<C>;
}

declare interface Chain<A> extends Apply<A> {
    chain<B>(fn: (a: A) => Chain<B>): Chain<B>;
}

declare interface Functor<A> {
    map<B>(fn: (a: A) => B): Functor<B>;
}

declare class Identity<A> implements Setoid, Semigroup, Monad<A> {
    private _value;
    constructor(value: A);
    static of<C>(v: C): Identity<C>;
    of<C>(v: C): Identity<C>;
    getValue(): A;
    equals(j: Identity<A>): boolean;
    concat(i: Identity<Semigroup>): Identity<A>;
    ap<B, C>(i: Identity<B>): Identity<C>;
    map<B>(f: (a: A) => B): Identity<B>;
    chain<B>(f: (a: A) => Identity<B>): Identity<B>;
    toString(): string;
}
export default Identity;

declare interface Monad<A> extends Applicative<A>, Chain<A> {
    chain<B>(fn: (a: A) => Monad<B>): Monad<B>;
}

declare interface Semigroup {
    concat(s: Semigroup): Semigroup;
}

declare interface Setoid {
    equals(s: Setoid): boolean;
}

export { }
