declare interface Alt<A> extends Functor<A> {
    alt(a: Alt<A>): Alt<A>;
}

declare interface Applicative<A> extends Apply<A> {
    of(a: A): Applicative<A>;
}

declare interface Apply<A> extends Functor<A> {
    ap<B, C>(j: Apply<B>): Apply<C>;
}

declare interface BiFunctor<A, B> extends Functor<B> {
    bimap<C, D>(fa: (a: A) => C, fb: (a: B) => D): BiFunctor<C, D>;
}

declare interface Chain<A> extends Apply<A> {
    chain<B>(fn: (a: A) => Chain<B>): Chain<B>;
}

declare abstract class Either<A, B> implements Setoid, BiFunctor<A, B>, Monad<B>, PatternMatch {
    protected _value: A | B;
    static of<C, D>(v: D): Either<C, D>;
    static Right<C, D>(v: D): Either<C, D>;
    static Left<C, D>(v: C): Either<C, D>;
    static fromNullable<C, D>(v: any): Either<C, D>;
    static withDefault<C, D, F>(def: any, v: Either<F, D>): Either<C, D>;
    static swap<C, D>(e: Either<C, D>): Either<C, D>;
    static try(f: Function): <C, D>(...args: Array<any>) => Either<C, D>;
    static bimap<G, K, C, D>(e: Either<G, K>, fl: (a: G) => C, fr: (b: K) => D): Either<C, D>;
    static isLeft<C, D>(v: Either<C, D>): boolean;
    static isRight<C, D>(v: Either<C, D>): boolean;
    of<C>(v: C): Either<A, C>;
    ap<C, D>(j: Either<C, B>): Either<C, D>;
    getValue(): A | B;
    abstract equals(n: Setoid): boolean;
    abstract map<C>(f: (a: B) => C): Either<A, C>;
    abstract bimap<C, D>(fl: (a: A) => C, fr: (a: B) => D): Either<C, D>;
    abstract chain<C>(f: (a: B) => Either<A, C>): Either<A, C>;
    abstract caseOf(o: {
        Left: (a: any) => any;
        Right: (a: any) => any;
    }): any;
    abstract swap(): Either<A, B>;
    abstract isLeft(): boolean;
    abstract isRight(): boolean;
}
export default Either;

declare interface Functor<A> {
    map<B>(fn: (a: A) => B): Functor<B>;
}

declare abstract class Maybe<A> implements Setoid, Monad<A>, Alt<A>, PatternMatch {
    protected _value: A;
    static of<B>(v: B): Maybe<B>;
    static zero<B>(): Maybe<B>;
    static Just<B>(v: B): Maybe<B>;
    static Nothing<B>(): Maybe<B>;
    static fromNullable<B>(v: any): Maybe<B>;
    static withDefault<B>(def: any, v: any): Maybe<B>;
    static catMaybes<B>(ar: Array<Maybe<B>>): Array<any>;
    static isNothing<B>(v: Maybe<B>): boolean;
    static isJust<B>(v: Maybe<B>): boolean;
    of<B>(v: B): Maybe<B>;
    alt<B>(v: Maybe<B>): Maybe<B>;
    ap<B, C>(j: Maybe<B>): Maybe<C>;
    getValue(): A;
    abstract equals(n: Setoid): boolean;
    abstract map<B>(f: (a: A) => B): Maybe<B>;
    abstract chain<B>(f: (a: A) => Maybe<B>): Maybe<B>;
    abstract caseOf(o: {
        Nothing: (a: any) => any;
        Just: (a: any) => any;
    }): any;
    abstract isNothing(): boolean;
    abstract isJust(): boolean;
}

export declare const maybeToEither: <A>(m: Maybe<A>) => any;

declare interface Monad<A> extends Applicative<A>, Chain<A> {
    chain<B>(fn: (a: A) => Monad<B>): Monad<B>;
}

declare interface PatternMatch {
    caseOf(o: {
        [k: string]: (a: any) => any;
    }): any;
}

declare interface Setoid {
    equals(s: Setoid): boolean;
}

export { }


declare namespace State {
    var of: <C, D>(v: C) => StateType<unknown, unknown>;
    var get: (fn?: Function) => StateType<any, any>;
    var put: (s: any) => StateType<any, any>;
}


declare namespace Reader {
    var of: <C>(v: C) => ReaderType<unknown, unknown>;
    var ask: (fn?: Function) => ReaderType<any, any>;
}


declare namespace Pair {
    var of: <C>(v: C) => PairType<C, C>;
}


declare namespace Identity {
    var of: <C>(v: C) => IdentityType<C>;
}
