declare interface Applicative<A> extends Apply<A> {
    of(a: A): Applicative<A>;
}

declare interface Apply<A> extends Functor<A> {
    ap<B, C>(j: Apply<B>): Apply<C>;
}

declare interface BiFunctor<A, B> extends Functor<B> {
    bimap<C, D>(fa: (a: A) => C, fb: (a: B) => D): BiFunctor<C, D>;
}

declare interface Chain<A> extends Apply<A> {
    chain<B>(fn: (a: A) => Chain<B>): Chain<B>;
}

declare interface Functor<A> {
    map<B>(fn: (a: A) => B): Functor<B>;
}

declare interface Monad<A> extends Applicative<A>, Chain<A> {
    chain<B>(fn: (a: A) => Monad<B>): Monad<B>;
}

declare class Pair<A, B> implements Setoid, Semigroup, BiFunctor<A, B>, Monad<B> {
    private _value;
    constructor(v1: A, v2: B);
    static of<C>(v: C): Pair<C, C>;
    of<C>(v: C): Pair<C, C>;
    equals(j: Pair<A, B>): boolean;
    concat(p: Pair<Semigroup, Semigroup>): Pair<A, B>;
    fst(): A;
    snd(): B;
    ap<C, D>(j: Pair<C, B>): Pair<C, D>;
    getValue(): (A | B)[];
    map<C>(f: (a: B) => C): Pair<A, C>;
    bimap<C, D>(f1: (a: A) => C, f2: (a: B) => D): Pair<C, D>;
    chain<C, D>(f: (a: B) => Pair<C, D>): Pair<C, D>;
    swap(): Pair<A | B, A | B>;
    toString(): string;
}
export default Pair;

declare interface Semigroup {
    concat(s: Semigroup): Semigroup;
}

declare interface Setoid {
    equals(s: Setoid): boolean;
}

export { }
