declare interface Applicative<A> extends Apply<A> {
    of(a: A): Applicative<A>;
}

declare interface Apply<A> extends Functor<A> {
    ap<B, C>(j: Apply<B>): Apply<C>;
}

declare interface Chain<A> extends Apply<A> {
    chain<B>(fn: (a: A) => Chain<B>): Chain<B>;
}

declare interface Functor<A> {
    map<B>(fn: (a: A) => B): Functor<B>;
}

declare interface Monad<A> extends Applicative<A>, Chain<A> {
    chain<B>(fn: (a: A) => Monad<B>): Monad<B>;
}

declare class Reader<A, B> implements Monad<B> {
    private _value;
    constructor(fn: Function);
    static of<C>(v: C): Reader<unknown, unknown>;
    of<C>(v: C): Reader<unknown, unknown>;
    ask(fn?: Function): Reader<A, B>;
    runWith(ctx: A): any;
    ap<C, D>(j: Reader<C, B>): Reader<C, D>;
    getValue(): Function;
    map<C>(f: Function): Reader<A, C>;
    chain<C, D>(f: (a: B) => Reader<C, D>): Reader<C, D>;
    toString(): string;
}
export default Reader;

export { }
