declare interface Applicative<A> extends Apply<A> {
    of(a: A): Applicative<A>;
}

declare interface Apply<A> extends Functor<A> {
    ap<B, C>(j: Apply<B>): Apply<C>;
}

declare interface Chain<A> extends Apply<A> {
    chain<B>(fn: (a: A) => Chain<B>): Chain<B>;
}

declare interface Functor<A> {
    map<B>(fn: (a: A) => B): Functor<B>;
}

declare interface Monad<A> extends Applicative<A>, Chain<A> {
    chain<B>(fn: (a: A) => Monad<B>): Monad<B>;
}

declare class State<A, B> implements Monad<B> {
    private _value;
    constructor(fn: Function);
    static of<G, K>(v: G): State<unknown, unknown>;
    of<C, D>(v: C): State<unknown, unknown>;
    get(fn?: Function): State<A, B>;
    put(s: B): State<A, B>;
    runWith(s: A): any;
    execWith(s: A): any;
    evalWith(s: A): any;
    ap<C, D>(j: State<C, (b: B) => D>): State<C, D>;
    getValue(): Function;
    map<C>(f: Function): State<A, C>;
    chain<C, D>(f: (a: B) => State<C, D>): State<C, D>;
    toString(): string;
}
export default State;

export { }
