declare interface Applicative<A> extends Apply<A> {
    of(a: A): Applicative<A>;
}

declare interface Apply<A> extends Functor<A> {
    ap<B, C>(j: Apply<B>): Apply<C>;
}

declare interface Chain<A> extends Apply<A> {
    chain<B>(fn: (a: A) => Chain<B>): Chain<B>;
}

declare interface Functor<A> {
    map<B>(fn: (a: A) => B): Functor<B>;
}

declare interface Monad<A> extends Applicative<A>, Chain<A> {
    chain<B>(fn: (a: A) => Monad<B>): Monad<B>;
}

declare interface Semigroup {
    concat(s: Semigroup): Semigroup;
}

declare class Task implements Semigroup, Monad<(rej: Function, res: Function) => any> {
    private _value;
    constructor(f: (rej: Function, res: Function) => any);
    static of(v: any): Task;
    static rejected(v: any): Task;
    of(v: any): Task;
    rejected(v: any): Task;
    fork(reject: Function, resolve: Function): void;
    toString(): string;
    map<A, B>(f: (a: A) => B): Task;
    getValue(): (rej: Function, res: Function) => any;
    ap(t: Task): Task;
    concat(t: Task): Task;
    chain(f: Function): Task;
    toPromise(): Promise<unknown>;
}
export default Task;

export { }
